#!/usr/bin/perl
#####################################################
#                                                   #
# PerlServ v1.0.1                                   #
# Written By: Samuel Sparling (samuel@wilter.com)   #
#                                                   #
# A simple web server designed to serve up text and #
# HTML documents. Nothing more than the simple GET  #
# method is used. This version will serve up binary #
# files as well.                                    #
#                                                   #
# 1.0 Released:   02-02-1999                        #
# 1.0.1 Released: 06-13-1999                        #
#                                                   #
# Released as open source. Feel free to make any    #
# modifications you like. Just give me credit where #
# it's due.                                         #
#                                                   #
# Tested On:                                        #
# Windows 95 w/ ActiveState Perl 5.005_02 Build 509 #
# SunOS 5.6 w/ perl 5.004_04                        #
# RedHat Linux 5.1 w/ perl 5.004_04                 #
#                                                   #
#####################################################
use Socket;# use the socket module

# The root directory of this web server.
$root_dir = "/htdocs";

# The default file when no file is specified.
$default = "index.html";

# The logfile this server will use.
$logfile = "/htdocs/log.txt";

# The port to run on.
$port = 80;

# Associative array of mime types
%mimes = ("html","text/html",
	"htm","text/html",
	"txt","text/plain",
	"gif","image/gif",
	"jpg","image/jpeg",
	"zip","application/x-zip-compressed",
	"tar","application/x-tar",
	"gz","application/x-gzip",);

# What to default to for unknown file extentions
$default_mime = "text/html";

#####################################################
# NO NEED TO EDIT BELOW THIS LINE.                  #
#####################################################

$proto = getprotobyname('tcp');
socket(Server, PF_INET, SOCK_STREAM, $proto) || die("socket: $!");
setsockopt(Server, SOL_SOCKET, SO_REUSEADDR, pack("l", 1)) || die("setsockopt: $!");
bind(Server, sockaddr_in($port, INADDR_ANY)) || die("bind: $!");
listen(Server, SOMAXCONN) || die("listen: $!");

$time=scalar(localtime);

&log("startup - PerlServ started on port $port - [$time]");

	for(; $paddr = accept(CLIENT,Server); close CLIENT)
	{
		($port,$iaddr) = sockaddr_in($paddr);
		$name = gethostbyaddr($iaddr,AF_INET);

		$get_request=<CLIENT>;
		# chop CR, LF, and CTRL+M off of the get request (for pretty log files)
		chop($get_request) if $get_request =~ /\r$/;
		chop($get_request) if $get_request =~ /\n$/;
		chop($get_request) if $get_request =~ /\cM/;

		($get,$file,$crap) = split(/\s+/, $get_request);
		$file .= "$default" if $file =~ m@/$@;

		@dot_parts=split(/\./,$file);
		$type=@dot_parts[int(@dot_parts - 1)];

		if($file =~ /\.\./)
		{
			$time=scalar(localtime);
			&log("$name - [$time] - \"$get_request\"");
			&error("Bad Request");
		}
		$time=scalar(localtime);

		&log("$name - [$time] - \"$get_request\"");
		open(FILE,"$root_dir$file") || &four_oh_four_error($file);
		binmode(FILE) unless $mimes{$type} =~ /^text/i;
		print CLIENT "HTTP/0.9 200 OK\r\n";
		print CLIENT "Date: $time\r\n";
		print CLIENT "Server: PerlServ v1.0.1\r\n";
		print CLIENT "Content-type: $mimes{$type}\r\n\r\n" if($type ne "");
		print CLIENT "Content-type: $default_mime\r\n\r\n" if($type eq "");
		while(<FILE>)
		{
			print CLIENT $_;
		}
		close(FILE);
		close(CLIENT);

	}

sub four_oh_four_error
{
	$file=shift(@_);
	print CLIENT "HTTP/0.9 404 Not Found\r\n\r\n";
	print CLIENT "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n";
	print CLIENT "<HTML><HEAD>\n";
	print CLIENT "<TITLE>404 Not Found</TITLE>\n";
	print CLIENT "</HEAD><BODY>\n";
	print CLIENT "<H1>Not Found</H1>\n";
	print CLIENT "The requested URL $file was not found on this server.<P>\n";
	print CLIENT "</BODY></HTML>\n";
	close(CLIENT);
}

sub error
{
	$error=shift(@_);
	print CLIENT "HTTP/0.9 400 Bad Request\r\n\r\n";
	print CLIENT "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n";
	print CLIENT "<HTML><HEAD>\n";
	print CLIENT "<TITLE>Error - $error</TITLE>\n";
	print CLIENT "</HEAD><BODY>\n";
	print CLIENT "<H1>$error</H1>\n";
	print CLIENT "There was an error in your request.<P>\n";
	print CLIENT "</BODY></HTML>\n";
	close(CLIENT);
}

sub log
{
	$msg=shift(@_);
	open(LOG,">>$logfile") || die($!);
	print LOG "$msg\n";
	close(LOG);
	print "$msg\n";
}
